using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using Benday.TfsUtility.JsonMessageTypes;
using System.IO;
using System.Text;
namespace Benday.TfsUtility.Tests
{
    [TestClass]
    public class BuildDefinitionExportCommandFixture : UnitTestBase
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
            Args = null;
        }
        public string[] Args { get; set; }

        private BuildDefinitionExportCommand _SystemUnderTest;
        public BuildDefinitionExportCommand SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest =
                        new BuildDefinitionExportCommand(Args);
                }

                return _SystemUnderTest;
            }
        }

        public string CommandNameArgument
        {
            get { return TfsUtilityConstants.CommandArgumentNameBuildDefinitionExport; }
        }

        public string BuildDefinitionName
        {
            get { return "BuildXyz"; }
        }

        [TestMethod]
        public void BuildDefinitionExportCommand_ValidateArgumentsSucceedsWithValidArguments()
        {
            Args = CreateArgsArray(
                CommandNameArgument,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName),
                GetArgEntry(TfsUtilityConstants.ArgumentNameBuildDefinitionName, BuildDefinitionName));

            Assert.IsNotNull(SystemUnderTest);
        }

        [TestMethod]
        public void BuildDefinitionExportCommand_ValidateArgumentsSucceedsWithValidArgumentsPlusFilename()
        {
            Args = CreateArgsArray(
                CommandNameArgument,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName),
                GetArgEntry(TfsUtilityConstants.ArgumentNameBuildDefinitionName, BuildDefinitionName),
                GetArgEntry(TfsUtilityConstants.ArgumentNameFilename, "yadayadayada.txt"));

            Assert.IsNotNull(SystemUnderTest);
        }
        
        [TestMethod]
        [ExpectedException(typeof(MissingArgumentException))]
        public void BuildDefinitionExportCommand_ValidateArgumentsFailsWithInvalidArguments()
        {
            Args = CreateArgsArray();

            Assert.IsNotNull(SystemUnderTest);
        }

        [TestMethod]
        public void BuildDefinitionExportCommand_GetResult()
        {
            Args = CreateArgsArray(
                CommandNameArgument,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName),
                GetArgEntry(TfsUtilityConstants.ArgumentNameBuildDefinitionName, BuildDefinitionName));

            var result = SystemUnderTest.GetResult();

            UnitTestUtility.AssertIsNotNullOrEmptyString(result, "result");
        }
    }
}
