using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using Benday.TfsUtility.JsonMessageTypes;
using System.IO;
using System.Text;
using System.Linq;

namespace Benday.TfsUtility.Tests
{
    [TestClass]
    public class BuildDefinitionImportCommandFixture : UnitTestBase
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
            Args = null;
        }
        public string[] Args { get; set; }

        private BuildDefinitionImportCommand _SystemUnderTest;
        public BuildDefinitionImportCommand SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest =
                        new BuildDefinitionImportCommand(Args);
                }

                return _SystemUnderTest;
            }
        }

        public string CommandNameArgument
        {
            get { return TfsUtilityConstants.CommandArgumentNameBuildDefinitionExport; }
        }

        public string BuildDefinitionNameToCreate
        {
            get { return "BuildXyz-uploaded"; }
        }

        public string GetUniqueBuildName()
        {
            string buildName = BuildDefinitionNameToCreate;

            var guid = Guid.NewGuid().ToString().Substring(0, 6);

            return buildName + "_" + guid;
        }

        public string FilenameForBuildDefinitionToUpload
        {
            get
            {
                return @"C:\code\Repos\misc-201312\Benday.TfsUtility\SampleBuildDefinition.json";
            }
        }

        [TestMethod]
        public void BuildDefinitionImportCommand_ValidateArgumentsSucceedsWithValidArguments()
        {
            Args = CreateArgsArray(
                CommandNameArgument,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName),
                GetArgEntry(TfsUtilityConstants.ArgumentNameBuildDefinitionName, BuildDefinitionNameToCreate),
                GetArgEntry(TfsUtilityConstants.ArgumentNameFilename, "yadayadayada.txt"));

            Assert.IsNotNull(SystemUnderTest);
        }
        
        [TestMethod]
        [ExpectedException(typeof(MissingArgumentException))]
        public void BuildDefinitionImportCommand_ValidateArgumentsFailsWithInvalidArguments()
        {
            Args = CreateArgsArray();

            Assert.IsNotNull(SystemUnderTest);
        }

        [TestMethod]
        public void BuildDefinitionImportCommand_UploadUnique()
        {
            var newBuildName = GetUniqueBuildName();

            Args = CreateArgsArray(
                CommandNameArgument,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName),
                GetArgEntry(TfsUtilityConstants.ArgumentNameBuildDefinitionName, newBuildName),
                GetArgEntry(TfsUtilityConstants.ArgumentNameFilename, FilenameForBuildDefinitionToUpload));

            SystemUnderTest.Upload();

            VerifyBuildWasUploaded(newBuildName);
        }

        [TestMethod]
        public void BuildDefinitionImportCommand_ReplaceExisting()
        {
            // arrange
            var newBuildName = GetUniqueBuildName();

            Args = CreateArgsArray(
                CommandNameArgument,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName),
                GetArgEntry(TfsUtilityConstants.ArgumentNameBuildDefinitionName, newBuildName),
                GetArgEntry(TfsUtilityConstants.ArgumentNameFilename, FilenameForBuildDefinitionToUpload));

            SystemUnderTest.Upload();

            // act (re-upload the build)
            SystemUnderTest.Upload();

            // assert
            VerifyBuildWasUploaded(newBuildName);
        }

        private void VerifyBuildWasUploaded(string expectedBuildName)
        {
            var command = new BuildDefinitionListCommand(Args);

            var result = command.GetResult();

            var match = (
                from temp in result
                where temp.Name == expectedBuildName
                select temp
                ).FirstOrDefault();

            Assert.IsNotNull(match, "Expected build '{0}' was not found.", expectedBuildName);
        }
    }
}
