using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using Benday.TfsUtility.JsonMessageTypes;
using System.IO;
using System.Text;
namespace Benday.TfsUtility.Tests
{
    [TestClass]
    public class BuildDefinitionListCommandFixture : UnitTestBase
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
            Args = null;
        }
        public string[] Args { get; set; }

        private BuildDefinitionListCommand _SystemUnderTest;
        public BuildDefinitionListCommand SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest =
                        new BuildDefinitionListCommand(Args);
                }

                return _SystemUnderTest;
            }
        }


        public string CommandNameArgument
        {
            get { return TfsUtilityConstants.CommandArgumentNameBuildDefinitionsList; }
        }
        

        [TestMethod]
        public void ValidateArgumentsSucceedsWithValidArguments()
        {
            Args = CreateArgsArray(
                CommandNameArgument,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName));

            Assert.IsNotNull(SystemUnderTest);
        }

        [TestMethod]
        public void ValidateArgumentsSucceedsWithValidArgumentsPlusFilter()
        {
            Args = CreateArgsArray(
                CommandNameArgument,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName));

            Assert.IsNotNull(SystemUnderTest);
        }

        [TestMethod]
        [ExpectedException(typeof(MissingArgumentException))]
        public void ValidateArgumentsFailsWithInvalidArguments()
        {
            Args = CreateArgsArray();

            Assert.IsNotNull(SystemUnderTest);
        }

        [TestMethod]
        public void GetResult()
        {
            Args = CreateArgsArray(
                CommandNameArgument,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName));

            var builds = SystemUnderTest.GetResult();

            Assert.IsNotNull(builds);
            Assert.AreNotEqual<int>(0, builds.Count, "Build count should not be zero.");

            JsonMessageTypeUtility.AssertBuildDefinitionInfoIsPopulated(builds[0]);
            JsonMessageTypeUtility.AssertBuildDefinitionInfoIsPopulated(builds[1]);
        }

        [TestMethod]
        public void RunCommandAndValidateConsoleOut_NameOnly()
        {
            Args = CreateArgsArray(
                CommandNameArgument,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName),
                GetArgEntry(TfsUtilityConstants.ArgumentNameNameOnly, String.Empty));

            StringBuilder expected = new StringBuilder();

            expected.AppendLine("Build1234");
            expected.AppendLine("BuildXyz");
            
            using (var writer = new StringWriter())
            {
                Console.SetOut(writer);

                SystemUnderTest.Run();

                var actual = writer.ToString();

                StringAssert.EndsWith(actual, expected.ToString());
            }
        }
    }
}
