﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;

namespace Benday.TfsUtility.Tests
{
    [TestClass]
    public class ListIterationsCommandFixture : UnitTestBase
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
            Args = null;
        }
        public string[] Args { get; set; }

        private ListIterationsCommand _SystemUnderTest;
        public ListIterationsCommand SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest =
                        new ListIterationsCommand(Args);
                }

                return _SystemUnderTest;
            }
        }

        public string SystemUnderTestCommandName
        {
            get { return TfsUtilityConstants.CommandArgumentNameListIterations; }
        }
        
        [TestMethod]
        public void ValidateArgumentsSucceedsWithValidArguments()
        {
            Args = CreateArgsArray(
                SystemUnderTestCommandName,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName));

            Assert.IsNotNull(SystemUnderTest);
        }

        [TestMethod]
        public void ValidateArgumentsSucceedsWithValidArgumentsPlusFilter()
        {
            Args = CreateArgsArray(
                SystemUnderTestCommandName,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName));

            Assert.IsNotNull(SystemUnderTest);
        }

        [TestMethod]
        [ExpectedException(typeof(MissingArgumentException))]
        public void ValidateArgumentsFailsWithInvalidArguments()
        {
            Args = CreateArgsArray();

            Assert.IsNotNull(SystemUnderTest);
        }

        [TestMethod]
        public void GetResultDefault()
        {
            Args = CreateArgsArray(
                SystemUnderTestCommandName,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName));

            List<string> result = SystemUnderTest.GetResult();

            Assert.AreNotEqual<int>(0, result.Count, "Result count is wrong.");

            result.ForEach(x => Console.WriteLine(x));
        }

        [TestMethod]
        public void GetResultForPathsOnly()
        {
            Args = CreateArgsArray(
                SystemUnderTestCommandName,
                GetArgEntry(TfsUtilityConstants.ArgumentNameTfsCollection, TfsCollectionUrl),
                GetArgEntry(TfsUtilityConstants.ArgumentNameTeamProject, TeamProjectName),
                GetArgEntry(TfsUtilityConstants.ArgumentNamePathsOnly, String.Empty));

            List<string> result = SystemUnderTest.GetResult();

            Assert.AreNotEqual<int>(0, result.Count, "Result count is wrong.");

            result.ForEach(x => Console.WriteLine(x));

            AssertContainsFolder(result, @"\scrum-git-20160101\Iteration\Release 1\Sprint 1");
            AssertContainsFolder(result, @"\scrum-git-20160101\Iteration\Sprint 1");            
        }
        
        private void AssertContainsFolder(List<string> actualFolderPaths, string expectedFolderPath)
        {
            Assert.IsTrue(actualFolderPaths.Contains(expectedFolderPath),
                String.Format("'{0}' was not in the result.", expectedFolderPath));
        }

        private void AssertDoesNotContainFolder(List<string> actualFolderPaths, string expectedFolderPath)
        {
            Assert.IsFalse(actualFolderPaths.Contains(expectedFolderPath),
                String.Format("'{0}' was in the result.", expectedFolderPath));
        }
    }
}
