using Microsoft.TeamFoundation.Server;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace Benday.TfsUtility
{
    public class AddIterationCommand : TfsCommandBase
    {
        public AddIterationCommand(string[] args)
            : base(args)
        {

        }

        protected override List<string> GetRequiredArguments()
        {
            var argumentNames = new List<string>();

            argumentNames.Add(TfsUtilityConstants.ArgumentNameTfsCollection);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameTeamProject);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameLevel1);

            return argumentNames;
        }

        protected override void DisplayUsage(StringBuilder builder)
        {
            base.DisplayUsage(builder);

            string usageString =
                String.Format("{0} {1} /collection:collectionurl /project:projectname /level1:name [/level2:name]",
                TfsUtilityConstants.ExeName,
                CommandArgumentName);

            builder.AppendLine(usageString);
        }

        protected override string CommandArgumentName
        {
            get { return TfsUtilityConstants.CommandArgumentNameAddIteration; }
        }


        public void Execute()
        {
            Connect();

            var project = FindProjectByName(
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject]);

            if (project == null)
            {
                RaiseExceptionAndDisplayError(true, "TFS project name does not exist.");
            }

            var commonStructureSvc = Tpc.GetService<ICommonStructureService4>();

            AddIterationPath(commonStructureSvc, project);
        }

        private void AddIterationPath(ICommonStructureService4 service, Project project)
        {
            string level1 = Arguments[TfsUtilityConstants.ArgumentNameLevel1];

            string level2 = null;

            if (Arguments.ContainsKey(TfsUtilityConstants.ArgumentNameLevel2) == true)
            {
                level2 = Arguments[TfsUtilityConstants.ArgumentNameLevel2];
            }

            AddIterationPath(service, project, level1, level2);
        }

        // private void CreateIterationPath(ICommonStructureService4 commonStructureSvc, string projectName, string iterationLevel1, string iterationLevel2)

        private void AddIterationPath(ICommonStructureService4 commonStructureSvc, Project project, string iterationLevel1, string iterationLevel2)
        {
            string projectName = project.Name;
            
            var rootNodePath = String.Format("\\{0}\\Iteration", projectName);

            NodeInfo rootNode = commonStructureSvc.GetNodeFromPath(rootNodePath);

            if (rootNode == null)
            {
                throw new InvalidOperationException("Could not find root node.");
            }
            
            var nodePathLevel1 = String.Format("\\{0}\\Iteration\\{1}", projectName, iterationLevel1);

            NodeInfo level1Node = null;

            try
            {
                level1Node = commonStructureSvc.GetNodeFromPath(nodePathLevel1);
            }
            catch (Exception ex)
            {
                if (ex.Message.Contains("The following node does not exist"))
                {
                    // don't worry
                }
                else
                {
                    throw ex;
                }
            }

            if (level1Node == null)
            {
                commonStructureSvc.CreateNode(iterationLevel1, rootNode.Uri);
                level1Node = commonStructureSvc.GetNodeFromPath(nodePathLevel1);
            }
            
            if (String.IsNullOrEmpty(iterationLevel2) == false)
            {
                var nodePathLevel2 = String.Format("\\{0}\\Iteration\\{1}\\{2}", projectName, iterationLevel1, iterationLevel2);

                NodeInfo level2Node = null;

                try
                {
                    level2Node = commonStructureSvc.GetNodeFromPath(nodePathLevel2);
                }
                catch (Exception ex)
                {
                    if (ex.Message.Contains("The following node does not exist"))
                    {
                        // don't worry
                    }
                    else
                    {
                        throw ex;
                    }
                }

                if (level2Node == null)
                {
                    commonStructureSvc.CreateNode(iterationLevel2, level1Node.Uri);
                    
                    level2Node = commonStructureSvc.GetNodeFromPath(nodePathLevel2);

                    if (level2Node == null)
                    {
                        throw new InvalidOperationException(
                            String.Format(
                            "Level 2 iteration node ('{0}') was still null after attempt to create it.",
                            nodePathLevel2));
                    }
                }                
            }
        }

        public override void Run()
        {
            Console.WriteLine();
            Execute();
            Console.WriteLine("Completed.");
        }
    }
}
