using Benday.TfsUtility.JsonMessageTypes;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization.Json;
using System.Text;

namespace Benday.TfsUtility
{
    public class BuildDefinitionListCommand : TfsCommandBase
    {
        public BuildDefinitionListCommand(string[] args)
            : base(args)
        {

        }

        protected override List<string> GetRequiredArguments()
        {
            var argumentNames = new List<string>();

            argumentNames.Add(TfsUtilityConstants.ArgumentNameTfsCollection);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameTeamProject);

            return argumentNames;
        }

        protected override void DisplayUsage(StringBuilder builder)
        {
            base.DisplayUsage(builder);

            string usageString =
                String.Format("{0} {1} /collection:collectionurl /project:projectname [/nameonly]",
                TfsUtilityConstants.ExeName,
                CommandArgumentName);

            builder.AppendLine(usageString);
        }

        protected override string CommandArgumentName
        {
            get { return TfsUtilityConstants.CommandArgumentNameBuildDefinitionsList; }
        }


        public List<BuildDefinitionInfo> GetResult()
        {
            string template = "{0}/{1}/_apis/build/definitions?api-version={2}";
            // https://{instance}/defaultcollection/{project}/_apis/build/definitions?api-version={version}[&name={string}][&type={string}]

            string requestUrl = String.Format(
                template,
                Arguments[TfsUtilityConstants.ArgumentNameTfsCollection],
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject],
                "2.0");

            using (WebClient client = new WebClient())
            {
                client.UseDefaultCredentials = true;

                string result = client.DownloadString(requestUrl);

                //dynamic json = JValue.Parse(result);

                var results =
                    JsonConvert.DeserializeObject<BuildDefinitionInfoResults>(result);

                return results.Values;
            }
        }
        
        public override void Run()
        {
            var results = GetResult();

            Console.WriteLine();

            Console.WriteLine("Result count: {0}", results.Count);

            results.ForEach(x => Console.WriteLine(ToString(x)));
        }

        private string ToString(BuildDefinitionInfo definition)
        {
            StringBuilder builder = new StringBuilder();

            builder.Append(definition.Name);

            if (Arguments.ContainsKey(TfsUtilityConstants.ArgumentNameNameOnly) == false)
            {
                builder.Append(" (");
                builder.Append(definition.Id);
                builder.Append(")");
            }
            
            return builder.ToString();
        }
    }
}
