using Benday.TfsUtility.JsonMessageTypes;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization.Json;
using System.Text;

namespace Benday.TfsUtility
{
    public class ReleaseDefinitionExportCommand : TfsCommandBase
    {
        public ReleaseDefinitionExportCommand(string[] args)
            : base(args)
        {

        }

        protected override List<string> GetRequiredArguments()
        {
            var argumentNames = new List<string>();

            argumentNames.Add(TfsUtilityConstants.ArgumentNameTfsCollection);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameTeamProject);
            argumentNames.Add(TfsUtilityConstants.ArgumentNameBuildDefinitionName);

            return argumentNames;
        }

        protected override void DisplayUsage(StringBuilder builder)
        {
            base.DisplayUsage(builder);

            string usageString =
                String.Format("{0} {1} /collection:collectionurl /project:projectname /{2}:buildname",
                TfsUtilityConstants.ExeName,
                CommandArgumentName, 
                TfsUtilityConstants.ArgumentNameReleaseDefinitionName);

            builder.AppendLine(usageString);
        }

        protected override string CommandArgumentName
        {
            get { return TfsUtilityConstants.CommandArgumentNameReleaseDefinitionExport; }
        }

        private string GetReleaseIdByName(string name)
        {
            string template = "{0}/{1}/_apis/release/definitions?api-version={2}&name={3}";
            // https://{instance}/defaultcollection/{project}/_apis/build/definitions?api-version={version}[&name={string}][&type={string}]

            string requestUrl = String.Format(
                template,
                Arguments[TfsUtilityConstants.ArgumentNameTfsCollection],
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject],
                "1.0",
                name);

            using (WebClient client = new WebClient())
            {
                client.UseDefaultCredentials = true;

                string result = client.DownloadString(requestUrl);

                var results =
                    JsonConvert.DeserializeObject<BuildDefinitionInfoResults>(result);

                if (results == null || results.Count == 0)
                {
                    return null;
                }
                else
                {
                    return results.Values[0].Id;
                }
            }
        }

        public string GetResult()
        {
            var name = Arguments[TfsUtilityConstants.ArgumentNameReleaseDefinitionName];

            string id = GetReleaseIdByName(name);

            if (id == null)
            {
                throw new InvalidOperationException(
                    String.Format("Release name '{0}' was not found.", name));
            }
            
            string template = "{0}/{1}/_apis/release/definitions/{2}?api-version={3}";
            
            string requestUrl = String.Format(
                template,
                Arguments[TfsUtilityConstants.ArgumentNameTfsCollection],
                Arguments[TfsUtilityConstants.ArgumentNameTeamProject],
                id, 
                "1.0");

            using (WebClient client = new WebClient())
            {
                client.UseDefaultCredentials = true;

                string result = client.DownloadString(requestUrl);

                return result;
            }
        }

        public override void Run()
        {
            if (ArgNameExists(TfsUtilityConstants.ArgumentNameFilename) == false)
            {
                Console.WriteLine();
                Console.WriteLine(GetResult());
            }
            else
            {
                Console.WriteLine();

                string filenameArgValue = Arguments[TfsUtilityConstants.ArgumentNameFilename];

                var filename = Path.Combine(Environment.CurrentDirectory, filenameArgValue);

                FileInfo info = new FileInfo(filename);

                var result = GetResult();

                if (String.IsNullOrWhiteSpace(result) == true)
                {
                    Console.WriteLine("Could not locate the definition.");
                }
                else
                {
                    File.WriteAllText(filename, result);

                    Console.WriteLine(String.Format("Definition written to '{0}'.", info.FullName));
                }
            }
        }
    }
}
